import os
from byper.constants import REQUIREMENTS_FILE
from ruamel.yaml import YAML


class Manifest:
    @staticmethod
    def save_manifest(data: dict, new_line: bool = False):
        out_data = {
            "name": data.get("name", os.path.basename(os.getcwd())),
            "dependencies": data.get("dependencies", {}),
            "scripts": data.get("scripts", {})
        }

        yaml = YAML()
        yaml.preserve_quotes = True
        yaml.indent(mapping=2, sequence=4, offset=2)
        yaml.width = 4096

        with open(REQUIREMENTS_FILE, "w") as f:
            yaml.dump({"name": out_data["name"]}, f)
            if out_data["scripts"]:
                f.write("\n")
                yaml.dump({"scripts": out_data["scripts"]}, f)
            if new_line:
                f.write("\n")
            if out_data["dependencies"]:
                yaml.dump({"dependencies": out_data["dependencies"]}, f)

    @staticmethod
    def load_requirements_manifest():
        if not os.path.exists(REQUIREMENTS_FILE):
            return {"name": "", "dependencies": {}, "scripts": {}}

        yaml = YAML()
        with open(REQUIREMENTS_FILE, "r") as f:
            data = yaml.load(f) or {}

        return {
            "name": data.get("name", ""),
            "dependencies": data.get("dependencies", {}),
            "scripts": data.get("scripts", {})
        }

    @staticmethod
    def load_script_from_manifest(_script: str):
        if not os.path.exists(REQUIREMENTS_FILE):
            return {}

        yaml = YAML()
        with open(REQUIREMENTS_FILE, "r") as f:
            data = yaml.load(f) or {}

        scripts: dict = data.get("scripts", {})
        return scripts.get(_script)



import importlib
import sys
from byper.constants import VERSION
from byper.core.commands import Commands
from byper.core.environment import Environment


def main():

    if len(sys.argv) > 1 and sys.argv[1].endswith(".py"):
        Environment.find_nested_venv()
        Commands.run_python_file(sys.argv[1])
        return

    parser = Commands.register_command()
    args = parser.parse_args()

    if args.command != "doctor":
        Environment.find_nested_venv()

    if args.command == "init":
        Commands.init(args.name)

    elif args.command == "add":
        for pkg in args.packages:
            Commands.add_package(pkg, args.no_cache)

    elif args.command == "tree":
        Commands.print_directory_tree()

    elif args.command == "run":
        Commands.run_script(args.script)

    elif args.version or args.command == "version":
        Logger = getattr(importlib.import_module(
            "byper.utils.logger"), "Logger"
        )

        Logger.log(VERSION, level="info")

    elif args.help or args.command == "help":
        Commands.print_help()

    elif args.command == "remove":
        for pkg in args.packages:
            Commands.remove_package(pkg)

    elif args.command == "login":
        Commands.login()

    elif args.command == "publish":
        Commands.publish()

    elif args.command == "doctor":
        Commands.doctor()

    else:
        if len(sys.argv) > 1:
            Commands.print_help()

        else:
            Commands.install()


if __name__ == "__main__":
    main()



import importlib
import os
from pathlib import Path
import subprocess
import sys
from byper.constants import ENVIRONMENT_DIRECTORY

Logger = getattr(importlib.import_module("byper.utils.logger"), "Logger")


class Environment:
    @staticmethod
    def find_nested_venv():
        other_envs = []
        for root, dirs, files in os.walk(os.getcwd()):
            rel_root = os.path.relpath(root, os.getcwd())

            # Skip Packages/ itself
            if rel_root == "Packages" or rel_root.startswith("Packages" + os.sep):
                continue

            is_env = (
                "pyvenv.cfg" in files or
                "site-packages" in dirs or
                "bin" in dirs or
                "Scripts" in dirs
            )

            if is_env:
                other_envs.append(rel_root)

        if other_envs:
            Logger.log("↪ Found additional virtual environment-like folders:", indent=1, level="warn")
            for path in other_envs:
                Logger.log(f"- {path}", indent=2, level="warn")
                
        return other_envs

    @staticmethod
    def get_python_version():
        return f"{sys.version_info.major}.{sys.version_info.minor}"

    @staticmethod
    def get_install_dir():
        return f"Packages/lib/python{Environment.get_python_version()}/site-packages"

    @staticmethod
    def get_env_python():
        return os.path.join(ENVIRONMENT_DIRECTORY, "bin", "python")

    @staticmethod
    def ensure_dirs(workspace: str = "./"):
        if not os.path.exists(workspace + ENVIRONMENT_DIRECTORY):
            subprocess.check_call([
                sys.executable, "-m", "venv", workspace + ENVIRONMENT_DIRECTORY
            ])

    @staticmethod
    def get_library_root():
        return Path(__file__).resolve().parent.parent

    @staticmethod
    def get_cache_dir():
        cache_dir = Environment.get_library_root() / ".cache/packages"
        cache_dir.mkdir(parents=True, exist_ok=True)

        return cache_dir



import argparse
import os
import importlib
from pathlib import Path
import shutil
import socket
import subprocess
import sys
import webbrowser
from colorama import Fore, Style
import pkg_resources
from byper.constants import REQUIREMENTS_FILE, ENVIRONMENT_DIRECTORY
from configparser import ConfigParser

Environment = getattr(importlib.import_module("byper.core.environment"), "Environment")
Manifest = getattr(importlib.import_module("byper.core.manifest"), "Manifest")
Logger = getattr(importlib.import_module("byper.utils.logger"), "Logger")
Installation = getattr(importlib.import_module("byper.core.installation"), "Installation")


class Commands:
    @staticmethod
    def doctor():
        Logger.log("🧠 Running environment diagnostics...")

        # Python version
        py_version = sys.version.split()[0]
        Logger.log(f"↪ Python version: {py_version}", indent=1, level="command")

        # Check for main 'Packages/' env
        packages_exists = os.path.isdir("Packages")

        # Check if environment is installed
        packages_exists = os.path.isdir("Packages")
        if packages_exists:
            Logger.log(f"↪ Python environment found at: {os.getcwd()}", indent=1, level="command")
        else:
            Logger.log("↪ Packages/ environment folder does not exist", indent=1, level="warn")

        # Internet access
        try:
            socket.create_connection(("pypi.org", 443), timeout=3)
            Logger.log("↪ Internet connectivity, successfully", indent=1, level="command")
        except Exception:
            Logger.log("↪ Internet connectivity, failed", indent=1, level="warn")

        # Warn about nested virtual environments (outside of Packages/)
        Environment.find_nested_venv()

        # Check for mismatched or broken packages
        broken_packages = []
        installed_packages = {dist.project_name: dist.version for dist in pkg_resources.working_set}

        for name, version in installed_packages.items():
            try:
                dist = pkg_resources.get_distribution(name)
                if dist.version != version:
                    broken_packages.append((name, version, dist.version))
            except Exception:
                broken_packages.append((name, version, "not installed"))

        if broken_packages:
            Logger.log("↪ Issues detected with installed packages:", indent=2, level="warn")
            for name, expected, actual in broken_packages:
                Logger.log(f"- {name}: expected {expected}, got {actual}", indent=3, level="warn")

        # Optional: print Python executable path
        Logger.log(f"↪ Python executable: {Environment.get_env_python()}", indent=1, level="command")

    @staticmethod
    def register_command():
        parser = argparse.ArgumentParser(add_help=False)
        subparsers = parser.add_subparsers(dest="command")

        parser.add_argument('-h', '--help', action='store_true', help='Show help')

        subparsers.add_parser("tree", help="Print directory tree")
        subparsers.add_parser("login", help="PyPI login")
        subparsers.add_parser("publish", help="Publish package to PyPI")
        subparsers.add_parser("doctor", help="Run dependencies diagnostics")

        parser.add_argument(
            "-v", "--version",
            help="Print byper version",
            action="store_true"
        )

        init_parser = subparsers.add_parser("init", help="Initialize byper project")
        init_parser.add_argument("name", nargs="?", default=None)

        add_parser = subparsers.add_parser("add", help="Add package to dependencies")
        add_parser.add_argument("packages", nargs="+")
        add_parser.add_argument(
            "--no-cache",
            action="store_true",
            help="Don't use cached packages",
        )

        run_parser = subparsers.add_parser("run", help="Run script")
        run_parser.add_argument("script")

        remove_parser = subparsers.add_parser("remove", help="Remove package from dependencies")
        remove_parser.add_argument("packages", nargs="+")

        return parser

    @staticmethod
    def print_help():
        Logger.log("List of byper commands and options:", level="info")
        Logger.log("Commands:", level="info")

        Logger.log("byper init                    Initialize byper project", indent=2, level="command")
        Logger.log("byper add <package-name>      Add package to dependencies", indent=2, level="command")
        Logger.log("byper tree                    Print directory tree", indent=2, level="command")
        Logger.log("byper run <script>            Run script", indent=2, level="command")
        Logger.log("byper remove <package-name>   Remove package from dependencies", indent=2, level="command")
        Logger.log("byper install                 Install dependencies", indent=2, level="command")
        Logger.log("byper                         Run byper by itself to install dependencies from Requirements", indent=2, level="command")

        Logger.log("\n")
        Logger.log("Options:", level="info")
        Logger.log("-h, --help                    Print help(byper -h, byper --help)", indent=2, level="command")
        Logger.log("--no-cache                    Install packages without use cached packages(byper add <package-name> --no-cache)", indent=2, level="command")

        exit()

    @staticmethod
    def print_directory_tree(start_path=".", prefix="", excluded_dirs={"Packages"}):
        try:
            entries = sorted(os.listdir(start_path))
        except OSError as e:
            print(f"{Fore.RED}Error accessing {start_path}: {e}")
            return

        files = []
        dirs = []

        for entry in entries:
            full_path = os.path.join(start_path, entry)
            if os.path.isdir(full_path):
                dirs.append(entry)
            else:
                files.append(entry)

        for index, directory in enumerate(dirs):
            is_last = index == len(dirs) - 1 and not files
            connector = "└── " if is_last else "├── "

            if directory in excluded_dirs:
                print(
                    f"{prefix}{connector}{Fore.LIGHTBLACK_EX}{directory}/*{Style.RESET_ALL}"
                )
            else:
                print(f"{prefix}{connector}{Fore.CYAN}{directory}/{Style.RESET_ALL}")
                new_prefix = f"{prefix}    " if is_last else f"{prefix}│   "
                Commands.print_directory_tree(
                    os.path.join(start_path, directory), new_prefix, excluded_dirs
                )

        for index, file in enumerate(files):
            is_last = index == len(files) - 1
            connector = "└── " if is_last else "├── "
            print(f"{prefix}{connector}{Fore.GREEN}{file}{Style.RESET_ALL}")

    @staticmethod
    def remove_package(package: str):
        try:
            Installation.uninstall(package)

        except ValueError as e:
            print(f"❌ {package} failed to remove: {e}")
            return

    @staticmethod
    def add_package(package, no_cache):
        try:
            Installation.reinstall_from_requirements()
            Installation.install(package, no_cache)

        except Exception as e:
            Logger.log(f"🗑️ {package} installation failed: {e}")

    @staticmethod
    def install():
        try:
            Installation.reinstall_from_requirements(True)

        except Exception as e:
            print(f"❌ {e}")

    @staticmethod
    def reinstall():
        Installation.reinstall_from_requirements()

    @staticmethod
    def init(name: str = None):
        if name:
            os.makedirs(name, exist_ok=True)
            os.chdir(name)

        if not os.path.exists(REQUIREMENTS_FILE):
            project_name = name or os.path.basename(os.getcwd())
            name = (
                input(f"Project name (default = {project_name}): ").strip()
                or project_name
            )

            entry = input("Entry point (default = main.py): ").strip() or "main.py"

            Manifest.save_manifest(
                {
                    "name": project_name,
                    "dependencies": {},
                    "scripts": {"start": f"python {entry}"},
                },
                True,
            )

            Environment.ensure_dirs()
            with open(entry, "w") as f:
                f.write("print('Hello, world!')")

            print(f"Initialized {REQUIREMENTS_FILE} environment in {os.getcwd()}")
        else:
            Logger.log(f"{REQUIREMENTS_FILE} manifest already exists in {os.getcwd()}")

            if not os.path.exists(ENVIRONMENT_DIRECTORY):
                Logger.log(f"📂 Creating missing {ENVIRONMENT_DIRECTORY} environment", indent=1, level="command")
                Logger.log(f"✅ {ENVIRONMENT_DIRECTORY} environment created", indent=1, level="success")
                Environment.ensure_dirs()

    @staticmethod
    def run_script(_script: str):
        script = Manifest.load_script_from_manifest(_script)
        if not script:
            print(f"Script '{_script}' not found in manifest.")
            return

        # Build environment for subprocess
        env_path = os.environ.copy()
        venv_bin = os.path.dirname(Environment.get_env_python())
        env_path["PATH"] = f"{venv_bin}:{env_path.get('PATH', '')}"
        env_path["VIRTUAL_ENV"] = os.path.abspath(os.path.dirname(venv_bin))

        subprocess.run(script, shell=True, env=env_path)

    @staticmethod
    def run_python_file(file_path: str):
        # Ensure it's a .py file
        if not file_path.endswith(".py"):
            Logger.log("❌ Only Python files (.py) can be executed.", level="debug")
            return

        # Get the path to the virtual environment's Python interpreter
        env_python = Environment.get_env_python()

        if not os.path.exists(env_python):
            Logger.log(f"❌ No byper environment found at {os.getcwd()}", level="debug")
            return

        # Build environment variables
        env_path = os.environ.copy()
        venv_bin = os.path.dirname(env_python)
        env_path["PATH"] = f"{venv_bin}{os.pathsep}{env_path.get('PATH', '')}"
        env_path["VIRTUAL_ENV"] = os.path.abspath(os.path.dirname(venv_bin))

        # Run the file using the virtual environment's Python
        try:
            subprocess.run([env_python, file_path], env=env_path, check=True)
        except subprocess.CalledProcessError as e:
            Logger.log(f"❌ Script exited with error: {e.returncode}")

    @staticmethod
    def login():
        Logger.log("🔐 PyPI Login Setup", newline=True, level="install")
        Logger.log(f"To upload packages to PyPI, you need an API token.")

        Logger.log(f"You can generate one here:")
        Logger.log(
            "→ https://pypi.org/manage/account/#api-tokens",
            indent=1,
            newline=True,
            level="command",
        )

        Logger.log("Opening link in your browser...", level="command")
        webbrowser.open("https://pypi.org/manage/account/#api-tokens")

        prompt = f"{Fore.BLUE}Enter your PyPI API token (starts with 'pypi-'): {Style.RESET_ALL}"
        token = input(prompt).strip()

        if not token.startswith("pypi-"):
            Logger.log(
                "⚠️ Warning: This doesn't look like a valid PyPI API token", level="warn"
            )
            return

        # Save token to ~/.pypirc
        pypirc_path = os.path.expanduser("~/.pypirc")
        config = ConfigParser()

        if os.path.exists(pypirc_path):
            config.read(pypirc_path)

        config["distutils"] = {"index-servers": "pypi"}
        config["pypi"] = {"username": "__token__", "password": token}

        with open(pypirc_path, "w") as f:
            config.write(f)

        Logger.log(
            f"\n✅ Login successful. Your PyPI token has been saved to {pypirc_path}.",
            level="success",
            newline=True,
        )

    @staticmethod
    def publish(dist_dir="dist"):
        project_root = Path.cwd()
        dist_path = project_root / dist_dir
        pypirc = Path.home() / ".pypirc"

        Logger.log("🚀 Preparing to upload your package to PyPI...\n")

        if not pypirc.exists():
            Logger.log(
                "❌ PyPI credentials not found. Please run the login command first."
            )
            return

        if (
            not (project_root / "setup.py").exists()
            and not (project_root / "pyproject.toml").exists()
        ):
            Logger.log("❌ No setup.py or pyproject.toml found. Cannot proceed.")
            return

        # Clean previous dist
        if dist_path.exists():
            shutil.rmtree(dist_path)

        Logger.log("📦 Building distribution using global Python...")

        try:
            subprocess.run(["python", "-m", "build"], check=True)
        except subprocess.CalledProcessError:
            Logger.log(
                "❌ Build failed. Make sure `build` is installed globally (e.g. pip install build)",
                level="error",
            )
            return

        Logger.log("📤 Uploading via twine using global Python...")

        try:
            subprocess.run(
                ["python", "-m", "twine", "upload", f"{dist_path}/*"],
                check=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
            )
            Logger.log("✅ Successfully uploaded to PyPI!", level="success")

        except subprocess.CalledProcessError as e:
            Logger.log("❌ Upload failed.", level="error")
            if e.stdout:
                Logger.log(e.stdout.decode(), level="error")



import importlib
import subprocess
import requests
from typing import TYPE_CHECKING
from packaging.requirements import Requirement
from packaging.version import Version
from packaging.specifiers import SpecifierSet


Environment = getattr(importlib.import_module("byper.core.environment"), "Environment")
Manifest = getattr(importlib.import_module("byper.core.manifest"), "Manifest")
Logger = getattr(importlib.import_module("byper.utils.logger"), "Logger")


class Installation:
    @staticmethod
    def install(package: str, no_cache: bool) -> str | None:
        try:
            Environment.ensure_dirs()

            manifest = Manifest.load_requirements_manifest()
            name, version = Installation.resolve_installable_version(package)
            is_installed = Installation.is_package_installed(name)

            Logger.log(f"\n📦 Installing {package}")

            # Install package using env Python
            version_to_install = f"=={version}" if version else ""
            args = list(filter(None, [
                Environment.get_env_python(),
                "-m",
                "pip",
                "install",
                f"{name}{version_to_install}",
                "--disable-pip-version-check",
                "--no-cache-dir" if no_cache else None
            ]))

            process = subprocess.Popen(
                args,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True
            )

            last_line = ""
            for line in process.stdout:
                last_line = line.strip()

                if "Successfully installed" in last_line:
                    Logger.log(
                        f"✅ {line.strip()}",
                        level="success",
                        indent=1
                    )

                else:
                    Logger.log(line.strip(), level="command", indent=1)

            if is_installed:
                Logger.log(
                    f"✅ {name}=={version} is already installed.",
                    level="install",
                    indent=1
                )

            if process.stderr:
                for line in process.stderr:
                    Logger.log(
                        f"❌ {line.strip()}",
                        level="error",
                        indent=1
                    )

            process.wait()

            manifest.setdefault("dependencies", {})[name] = version
            Manifest.save_manifest(manifest)

            return name, version

        except Exception as e:
            return None, None

    @staticmethod
    def reinstall_from_requirements(show_logdown: bool = False, no_cache: bool = False):
        try:
            Environment.ensure_dirs()
            manifest = Manifest.load_requirements_manifest()
            dependencies = dict(manifest.get("dependencies", {}))

            if not dependencies:
                return

            if show_logdown and len(dependencies) > 0:
                Logger.log(
                    f"📦 Reinstalling dependencies",
                    level="install"
                )

            for m_package, version in dependencies.items():
                is_installed = Installation.is_package_installed(
                    m_package
                )

                if not is_installed:
                    package_to_install = f"{m_package}=={version}"
                    Installation.install(package_to_install, no_cache)

                else:
                    if show_logdown:
                        Logger.log(
                            f"✅ {m_package} is already installed.",
                            level="command",
                            indent=1
                        )

        except subprocess.CalledProcessError:
            raise Exception

    @staticmethod
    def uninstall(package: str) -> str | None:
        try:
            Environment.ensure_dirs()
            name, _ = Installation.resolve_installable_version(package)

            Logger.log(
                f"\n📦 Uninstalling {package}",
                level="install"
            )

            process = subprocess.Popen(
                [
                    Environment.get_env_python(),
                    "-m",
                    "pip",
                    "--disable-pip-version-check",
                    "uninstall",
                    "-y",
                    f"{name}"
                ],
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True
            )

            last_line = ""
            for line in process.stdout:
                last_line = line.strip()

                if "Successfully uninstalled" in last_line:
                    Logger.log(
                        f"❌ {line.strip()}",
                        level="remove",
                        indent=1
                    )

                else:
                    Logger.log(line.strip(), level="command", indent=1)

            if process.stderr:
                for line in process.stderr:
                    Logger.log(
                        f"❌ {line.strip()}",
                        level="error",
                        indent=1
                    )

            process.wait()

            manifest = Manifest.load_requirements_manifest()
            dependencies: dict = dict(manifest.get("dependencies", {}))

            if name in dependencies:
                del dependencies[name]

                manifest["dependencies"] = dependencies
                Manifest.save_manifest(manifest)

        except subprocess.CalledProcessError as e:
            raise ValueError(e.stderr)

    @staticmethod
    def is_package_installed(package: str):
        try:
            env_python = Environment.get_env_python()
            subprocess.run(
                [env_python, "-m", "pip", "show", package],
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL,
                text=True,
                check=True
            )

            return True

        except subprocess.CalledProcessError as e:
            return False

    @staticmethod
    def resolve_installable_version(requirement_str: str):
        requirement = Requirement(requirement_str)
        try:
            name = requirement.name
            specifier: SpecifierSet = requirement.specifier

            url = f"https://pypi.org/pypi/{name}/json"
            resp = requests.get(url)
            if resp.status_code != 200:
                return name, None

            all_versions = sorted(
                [Version(v) for v in resp.json()["releases"].keys()
                 if not Version(v).is_prerelease],
                reverse=True
            )

            compatible_versions = [str(v)
                                   for v in all_versions if v in specifier]
            if compatible_versions:
                return name, compatible_versions[0]
            else:
                return name, None

        except Exception:
            return requirement.name, None



from colorama import Fore, init

init(autoreset=True)


class Logger:
    @staticmethod
    def log(message, level="info", indent=0, newline=False):
        prefix = {
            "info": Fore.CYAN,
            "warn": Fore.YELLOW,
            "error": Fore.RED,
            "success": Fore.GREEN,
            "download": Fore.BLUE,
            "install": Fore.MAGENTA,
            "debug": Fore.WHITE,
            "step": Fore.CYAN,
            "replace": Fore.YELLOW,
            "skip": Fore.GREEN,
            "remove": Fore.RED,
            "command": Fore.BLACK,
        }.get(level, "")
        indent_space = "  " * indent
        print(f"{indent_space}{prefix}{message}",
              end="\n" if not newline else "\n\n")



import importlib
import os
from pathlib import Path
import subprocess
from byper.constants import LOCKFILE
from ruamel.yaml import YAML
import yaml

Environment = getattr(importlib.import_module("byper.core.environment"), "Environment")
Logger = getattr(importlib.import_module("byper.utils.logger"), "Logger")
Commands = getattr(importlib.import_module("byper.core.commands"), "Commands")


class Lockfile:
    @staticmethod
    def load_lockfile_manifest():
        if not os.path.exists(LOCKFILE):
            return {}

        yaml = YAML()
        with open(LOCKFILE, "r") as f:
            data = yaml.load(f) or {}

        packages = data.get("packages", {})
        return dict(packages)

    @staticmethod
    def write_lockfile(package: dict):
        lock_entry = {
            package["name"]: {
                "version": package["version"],
                "filename": package["filename"],
                "filetype": package["filetype"],
                "url": package["url"],
                "hash": package["hash"],
                "bin_files": package.get("bin_files", []),
                "dependencies": package.get("dependencies", []),
            }
        }

        yaml = YAML()
        yaml.preserve_quotes = True
        yaml.indent(mapping=2, sequence=4, offset=2)
        yaml.width = 4096

        # Load existing lockfile data if it exists
        if Path(LOCKFILE).exists():
            with open(LOCKFILE, "r") as f:
                data = yaml.load(f) or {}

        else:
            data = {}

        data.setdefault("packages", {}).update(lock_entry)

        with open(LOCKFILE, "w") as f:
            yaml.dump(data, f)

    @staticmethod
    def remove_from_lockfile(package_name: str):
        lockfile_path = Path(LOCKFILE)
        if not lockfile_path.exists():
            Logger.log("🔍 Lockfile does not exist.", level="warn")
            return

        with open(lockfile_path, "r") as f:
            try:
                data = yaml.safe_load(f) or {}
            except yaml.YAMLError as e:
                Logger.log(
                    f"🗑️ Failed to parse lockfile: {e}", level="error")
                return

        packages = data.get("packages", {})
        if package_name not in packages:
            return

        package = packages.get(package_name, {})
        bin_files = package.get("bin_files", [])

        # Remove installed bin files
        for bin_file in bin_files:
            file_path = f"{Environment.get_install_dir()}/bin/{bin_file}"
            if os.path.exists(file_path):
                os.remove(file_path)

        del packages[package_name]

        # Clean up if no packages left
        if not packages:
            data = {}
        else:
            data["packages"] = packages

        with open(lockfile_path, "w") as f:
            yaml.safe_dump(data, f, sort_keys=False)

    @staticmethod
    def install_from_lockfile():

        if not os.path.exists(LOCKFILE):
            print(f"Lockfile {LOCKFILE} not found.")
            return
        with open(LOCKFILE, "r") as f:
            lock_data = yaml.safe_load(f)
        packages = lock_data.get("package", {})
        for pkg, info in packages.items():
            version = info.get("version")
            try:
                Commands.add_package(f"{pkg}=={version}")
            except subprocess.CalledProcessError:
                print(f"Failed to install {pkg}=={version}")
        print(f"Installed dependencies from {LOCKFILE}")
